<?php

namespace FM\Factory;

use Doctrine\ORM\EntityManager;
use FM\Traits\TQuery;

class Fm00004001TransferirTarefasFactory
{
    use TQuery;

    protected $connection;
    protected $entityManager;

    public function __construct(EntityManager $entityManager)
    {
        $this->connection = $entityManager->getConnection();
        $this->entityManager = $entityManager;
        $this->alias = "checklist";
    }

    public function listTask($params)
    {
        $sql = "SELECT FCM_CHECKLISTAGEND.NRCHECKLISTAGEND, FCM_CHECKLISTAGEND.DTHORAAGEND, FCM_CHECKLISTAGEND.NMCHECKLISTAGEND, FCM_CHECKLISTAGEND.DTHRFINALAGEND,
                FCM_LOCALCLI.DSCORIDLOCAL, MAX(FCM_ATIVIDADE.DSCORIDATIVIDADE) AS CORATIVIDADE
                        FROM FCM_CHECKLISTAGEND
                        JOIN FCM_LOCALCLI
                          ON FCM_LOCALCLI.NRLOCALCLIENTE = FCM_CHECKLISTAGEND.NRLOCALCLIENTE
                        JOIN FCM_ATIVIDADECHK
                          ON FCM_ATIVIDADECHK.NRCHECKLIST = FCM_CHECKLISTAGEND.NRCHECKLIST
                        JOIN FCM_ATIVIDADE
                          ON FCM_ATIVIDADE.NRATIVIDADE = FCM_ATIVIDADECHK.NRATIVIDADE
                 WHERE TRUNC(FCM_CHECKLISTAGEND.DTHORAAGEND) BETWEEN TO_DATE( :startPeriod, 'YYYY-MM-DD') AND TO_DATE( :finishPeriod, 'YYYY-MM-DD')
                    AND FCM_CHECKLISTAGEND.CDOPERADOR = :CDOPERADOR " . $params . "
                    GROUP BY FCM_CHECKLISTAGEND.NRCHECKLISTAGEND, FCM_CHECKLISTAGEND.DTHORAAGEND, FCM_CHECKLISTAGEND.NMCHECKLISTAGEND, FCM_CHECKLISTAGEND.DTHRFINALAGEND, FCM_LOCALCLI.DSCORIDLOCAL
                    ORDER BY DTHORAAGEND";

        return $this->connection->executeQuery($sql, $params)->fetchAllAssociative();
    }

    public function infoTask($param)
    {

        $sql = "SELECT FCM_CHECKLISTAGEND.NMCHECKLISTAGEND, FCM_TIPOLOCALCLI.NMTIPOLOCALCLI, FCM_LOCALCLI.NMLOCALCLIENTE, OPERADOR.NMOPERADOR, FCM_TIPOLOCALCLI.NRTIPOLOCALCLI, FCM_CHECKLISTAGEND.NRLOCALCLIENTE, 
                    FCM_CHECKLISTAGEND.CDOPERADOR, FCM_CHECKLISTAGEND.DTHORAAGEND, FCM_CHECKLISTAGEND.DTHRFINALAGEND
                    FROM FCM_CHECKLISTAGEND
                    JOIN FCM_LOCALCLI
                        ON FCM_LOCALCLI.NRLOCALCLIENTE = FCM_CHECKLISTAGEND.NRLOCALCLIENTE
                    JOIN FCM_TIPOLOCALCLI
                        ON FCM_LOCALCLI.NRTIPOLOCALCLI = FCM_TIPOLOCALCLI.NRTIPOLOCALCLI
                    JOIN OPERADOR
                        ON FCM_CHECKLISTAGEND.CDOPERADOR = OPERADOR.CDOPERADOR
                    WHERE FCM_CHECKLISTAGEND.NRCHECKLISTAGEND = :NRCHECKLISTAGEND
                        AND FCM_CHECKLISTAGEND.CDOPERADOR = :CDOPERADOR";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $param);
    }

    public function informacoesVistoria($parametros)
    {

        $sql = " SELECT FCM_GERENCIAMENTOCHK.NMGERENCIAMENTOCHK, FCM_TIPOLOCALCLI.NMTIPOLOCALCLI, FCM_LOCALCLI.NMLOCALCLIENTE, OPERADOR.NMOPERADOR, FCM_TIPOLOCALCLI.NRTIPOLOCALCLI, FCM_GERENCIAMENTOCHK.NRLOCALCLIENTE, 
                    FCM_GERENCIAMENTOCHK.CDOPERADOR, FCM_GERENCIAMENTOCHK.DTHORAAGEND, FCM_GERENCIAMENTOCHK.DTHRFINALAGEND
                    FROM FCM_GERENCIAMENTOCHK
                    JOIN FCM_LOCALCLI
                      ON FCM_LOCALCLI.NRLOCALCLIENTE = FCM_GERENCIAMENTOCHK.NRLOCALCLIENTE
                    JOIN FCM_TIPOLOCALCLI
                      ON FCM_LOCALCLI.NRTIPOLOCALCLI = FCM_TIPOLOCALCLI.NRTIPOLOCALCLI
                    JOIN OPERADOR
                      ON FCM_GERENCIAMENTOCHK.CDOPERADOR = OPERADOR.CDOPERADOR
                    WHERE FCM_GERENCIAMENTOCHK.NRGERENCIAMENTOCHK = :NRGERENCIAMENTOCHK
                      AND FCM_GERENCIAMENTOCHK.CDOPERADOR = :CDOPERADOR";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $parametros);
    }

    public function taskOfTheEmployee($param, $periodoGer, $periodoCheck)
    {
        $sql = "SELECT ('V-' || FCM_GERENCIAMENTOCHK.NRGERENCIAMENTOCHK) as NRCHECKLISTAGEND,
                       ('Vistoria-' || FCM_GERENCIAMENTOCHK.NMGERENCIAMENTOCHK) as NMCHECKLISTAGEND, FCM_LOCALCLI.NMLOCALCLIENTE as NMLOCALCLIENTE,
                       FCM_GERENCIAMENTOCHK.DTHORAAGEND as DTHORAAGEND
                  FROM FCM_GERENCIAMENTOCHK
                  JOIN FCM_LOCALCLI
                    ON FCM_LOCALCLI.NRLOCALCLIENTE = FCM_GERENCIAMENTOCHK.NRLOCALCLIENTE
                WHERE FCM_GERENCIAMENTOCHK.CDOPERADOR = :nr
                {$periodoGer}

                UNION

                SELECT TO_CHAR(FCM_CHECKLISTAGEND.NRCHECKLISTAGEND) as NRCHECKLISTAGEND, FCM_CHECKLISTAGEND.NMCHECKLISTAGEND as NMCHECKLISTAGEND,
                FCM_LOCALCLI.NMLOCALCLIENTE as NMLOCALCLIENTE, FCM_CHECKLISTAGEND.DTHORAAGEND as DTHORAAGEND
                  FROM FCM_CHECKLISTAGEND
                  JOIN FCM_LOCALCLI
                    ON FCM_LOCALCLI.NRLOCALCLIENTE = FCM_CHECKLISTAGEND.NRLOCALCLIENTE
                WHERE FCM_CHECKLISTAGEND.CDOPERADOR = :nr
                {$periodoCheck}
              ORDER BY DTHORAAGEND";

        return $this->connection->executeQuery($sql, $param)->fetchAllAssociative();
    }

    public function transferTask($param, $nrchecklistagend)
    {
        $sql = "UPDATE FCM_CHECKLISTAGEND
                 SET CDOPERADOR = :CDOPERADOR,
                 DTULTATU = SYSDATE,
                 CDOPERULTATU = :CDOPERULTATU,
                 NRORGULTATU = :NRORGULTATU
               WHERE NRCHECKLISTAGEND IN (" . $nrchecklistagend . ")";

        $this->connection->executeStatement($sql, $param);
    }

    public function dadosTask($param)
    {
        $sql = "SELECT NRCHECKLISTAGEND, CDOPERADOR, DTHORAAGEND
              FROM FCM_CHECKLISTAGEND
             WHERE NRCHECKLISTAGEND = " . $param;

        return $this->connection->executeQuery($sql)->fetchAllAssociative();
    }

    public function getLocals($id)
    {

        $sql = "SELECT FCM_LOCALCLI.NRLOCALCLIENTE, FCM_LOCALCLI.NMLOCALCLIENTE,
                           FCM_LOCALCLI.NRSETOR, FCM_LOCALCLI.NRLOCALCLIENTEPAI, FCM_TIPOLOCALCLI.NRTIPOLOCALCLI
                     FROM FCM_LOCALCLI
                     JOIN FCM_TIPOLOCALCLI
                       ON FCM_LOCALCLI.NRTIPOLOCALCLI = FCM_TIPOLOCALCLI.NRTIPOLOCALCLI
                        WHERE FCM_TIPOLOCALCLI.IDTIPOLOCALCLI = :ID";


        return $this->connection->executeQuery($sql, $id)->fetchAllAssociative();
    }

    public function listChecklistAgend($params)
    {
        $useParams = false;
        $sqlCode = "SELECT checklist.NRCHECKLISTAGEND, checklist.NRLOCALCLIENTE,
            checklist.NRFUNCIONARIO, checklist.IDSITUACAOAGEND, checklist.DTHORAAGEND,
            checklist.IDATIVO, checklist.NRORG, checklist.DTINCLUSAO,
            checklist.CDOPERINCLUSAO, checklist.NRORGINCLUSAO, checklist.DTULTATU,
            checklist.CDOPERULTATU, checklist.NRORGULTATU, checklist.NMCHECKLISTAGEND,
            checklist.DSJUSTCANCELAMENTO FROM FCM_CHECKLISTAGEND checklist";
        $whereClausule =  $this->concatWhereClausule($params);
        if (strlen($whereClausule) > 0) {
            $useParams = true;
            $sqlCode .= $whereClausule;
        }
        $std = $this->connection->prepare($sqlCode);
        if ($useParams) {
            $this->connection->executeStatement($params);
        } else {
            $this->connection->executeStatement($std);
        }
        return $this->connection->executeQuery($std)->fetchAllAssociative();
    }

    private function concatWhereClausule($params)
    {
        $useParams = false;
        $paramKeys = array_keys($params);
        $query = "";
        foreach ($paramKeys as $key) {
            if (!$useParams) {
                $useParams = true;
                $query .= " WHERE ";
            } else {
                $query .= " AND ";
            }
            $query .= $this->alias . "." . $key . " = :" . $key;
        }
        return $query;
    }


    public function checklistAgendsInnerJoinChecklist($params)
    {
        $useParams = false;
        $sqlCode = "SELECT checklist.NRCHECKLISTAGEND, checklist.NRLOCALCLIENTE,
            checklist.NRCHECKLIST,
            checklist.NRFUNCIONARIO, checklist.IDSITUACAOAGEND, checklist.DTHORAAGEND,
            checklist.IDATIVO, checklist.NRORG, checklist.DTINCLUSAO,
            checklist.CDOPERINCLUSAO, checklist.NRORGINCLUSAO, checklist.DTULTATU,
            checklist.CDOPERULTATU, checklist.NRORGULTATU, checklist.NMCHECKLISTAGEND,
            checklist.DSJUSTCANCELAMENTO, chklist.NRSETOR, local.DSCORIDLOCAL,
            checklist.DTHRFINALAGEND, chklist.DSCHECKLIST, atv.DSCORIDATIVIDADE
            FROM FCM_CHECKLISTAGEND checklist
            INNER JOIN FCM_CHECKLIST chklist
            ON checklist.NRCHECKLIST = chklist.NRCHECKLIST
            INNER JOIN FCM_LOCALCLI local
            ON local.NRLOCALCLIENTE = checklist.NRLOCALCLIENTE
            INNER JOIN FCM_ATIVIDADECHK atvchk
            ON atvchk.NRCHECKLIST = chklist.NRCHECKLIST
            INNER JOIN FCM_ATIVIDADE atv
            ON atv.NRATIVIDADE = atvchk.NRATIVIDADE";
        $paramKeys = array_keys($params);
        $query = "";
        $paramCount = 0;
        $useAnd = false;
        foreach ($paramKeys as $key) {
            $alias = "";
            if ($useAnd) {
                $query .= " AND ";
            } else {
                $query .= " WHERE ";
                $useAnd = true;
            }
            // os ifs a seguir servem para definir o apelido da tabela
            // a qual vai ser usada para comparar o atributo passado
            if ($key == "NRCHECKLIST") {
                $alias = "chklist";
            } elseif ($key == "NRTIPOLOCALCLI" || $key == "NRSETOR") {
                $alias = "local";
            } else {
                $alias = "checklist";
            }

            if (is_array($params[$key])) {
                $query .= $alias . "." . $key . " IN (";
                $lenParams = count($params[$key]);
                for ($i = 0; $i < $lenParams; $i++) {
                    $query .= $params[$key][$i];
                    if ($i !== $lenParams - 1) {
                        $query .= ", ";
                    }
                }
                $query .= ") ";
                unset($params[$key]);
            } else {
                $query .= $alias . "." . $key . " = :" . $key;
                $paramCount++;
            }
        }
        $sqlCode .= $query;
        $std = $this->connection->prepare($sqlCode);
        if ($paramCount > 0) {
            $this->connection->executeStatement($params);
        } else {
            $this->connection->executeStatement($std);
        }
        return $this->connection->executeQuery($std)->fetchAllAssociative();
    }

    public function updateChecklistAgend($params)
    {
        $idEntity = $params["NRCHECKLISTAGEND"];
        unset($params["NRCHECKLISTAGEND"]);
        $sql = "UPDATE FCM_CHECKLISTAGEND
        SET ";
        $query = $this->concatUpdate($params);
        $sql .= $query . " WHERE NRCHECKLISTAGEND = " . $idEntity;
        return $this->connection->executeQuery($sql, $params)->fetchAllAssociative();
    }

    protected function concatUpdate($params)
    {
        $paramKeys = array_keys($params);
        $query = "";
        for ($i = 0; $i < count($paramKeys); $i++) {
            $query .= $paramKeys[$i] . " = :" . $paramKeys[$i];
            if ($i != count($paramKeys) - 1) {
                $query .= ", ";
            }
        }
        return $query;
    }

    public function setDTHRCHECLIST()
    {
        $sql = "UPDATE FCM_CHECKLISTAGEND SET IDSITUACAOAGEND = 5 WHERE IDSITUACAOAGEND = 1 AND DTHORAAGEND < SYSDATE";
        $this->connection->executeStatement($sql);
    }
}
