<?php

namespace FM\Factory;

use Doctrine\ORM\EntityManager;
use FM\Traits\TQuery;

class VistoriasFactory
{

    use TQuery;
    protected $connection;
    protected $entityManager;

    public function __construct(EntityManager $entityManager)
    {
        $this->connection = $entityManager->getConnection();
        $this->entityManager = $entityManager;
    }

    // Busca nome e local da vistoria
    public function getInspectionNameAndLocal($params)
    {
        $sql = "SELECT A.NMGERENCIAMENTOCHK, A.NRGERENCIAMENTOCHK, B.NMLOCALCLIENTE
                    FROM FCM_GERENCIAMENTOCHK A
                        INNER JOIN FCM_LOCALCLI B ON B.NRLOCALCLIENTE = A.NRLOCALCLIENTE
                 WHERE A.NRGERENCIAMENTOCHK = {$params}";

        return $this->connection->executeQuery($sql, $params)->fetchAllAssociative();
    }

    // Busca e filtra vistorias
    public function getInspectionsInfo($hierarchyParam)
    {

        $sql = "SELECT GCHK.NRGERENCIAMENTOCHK, GCHK.NMGERENCIAMENTOCHK, GCHK.NRLOCALCLIENTE, GCHK.CDOPERADOR, GCHK.IDSITUACAOGERCHK,
                  TO_CHAR(GCHK.DTHORAAGEND, 'DD/MM HH24:MI')DTHORAAGEND, FUNC.NMOPERADOR, LC.NMLOCALCLIENTE, LC.NRLOCALCLIENTEPAI AS LOCALPAI, LCP.NMLOCALCLIENTE AS NMLOCALPAI, CA.IDSITUACAOAGEND,
                      DECODE(CA.IDSITUACAOAGEND, '4', CASE GCHK.IDSITUACAOGERCHK WHEN '1' THEN 'Aguardando Vistoria'
                                        WHEN '2' THEN 'Em andamento'
                                        WHEN '3' THEN 'Avaliado'
                                        ELSE NULL END, 'Tarefa Pendente') AS IDSITUACAOGERCHK
                          FROM FCM_GERENCIAMENTOCHK GCHK
                          INNER JOIN FCM_LOCALCLI LC
                          ON LC.NRLOCALCLIENTE = GCHK.NRLOCALCLIENTE
                          INNER JOIN OPERADOR FUNC
                          ON FUNC.CDOPERADOR = GCHK.CDOPERADOR
                          INNER JOIN FCM_LOCALCLI LCP
                          ON LCP.NRLOCALCLIENTE = LC.NRLOCALCLIENTEPAI
                          INNER JOIN FCM_CHECKLISTAGEND CA
                          ON CA.NRCHECKLISTAGEND = GCHK.NRCHECKLISTAGEND 
                    WHERE GCHK.CDOPERADOR IN ({$hierarchyParam})
                          ORDER BY DTHORAAGEND";

        return $this->connection->executeQuery($sql)->fetchAllAssociative();
    }

    public function getVistorias($hierarchyParam)
    {

        $sql = "SELECT GCHK.NRGERENCIAMENTOCHK, GCHK.NMGERENCIAMENTOCHK, GCHK.NRLOCALCLIENTE, GCHK.CDOPERADOR,
                            TO_CHAR(GCHK.DTHORAAGEND, 'DD/MM HH24:MI')DTHORAAGEND, FUNC.NMOPERADOR, LC.NMLOCALCLIENTE, LC.NRLOCALCLIENTEPAI AS LOCALPAI, LCP.NMLOCALCLIENTE AS NMLOCALPAI, CA.IDSITUACAOAGEND,
                            FCT_CONTRATO.NRCONTRATOFCT, DECODE(CA.IDSITUACAOAGEND, '4', CASE GCHK.IDSITUACAOGERCHK WHEN '1' THEN 'Aguardando Vistoria'
                                        WHEN '2' THEN 'Em andamento'
                                        WHEN '3' THEN 'Avaliado'
                                        ELSE NULL END, 'Tarefa Pendente') AS IDSITUACAOGERCHK
                        FROM FCM_GERENCIAMENTOCHK GCHK
                    INNER JOIN FCM_LOCALCLI LC
                            ON LC.NRLOCALCLIENTE = GCHK.NRLOCALCLIENTE
                    INNER JOIN OPERADOR FUNC
                            ON FUNC.CDOPERADOR = GCHK.CDOPERADOR
                    INNER JOIN FCM_LOCALCLI LCP
                            ON LCP.NRLOCALCLIENTE = LC.NRLOCALCLIENTEPAI
                    INNER JOIN FCM_CHECKLISTAGEND CA
                            ON CA.NRCHECKLISTAGEND = GCHK.NRCHECKLISTAGEND
                        JOIN FCT_CONTRATO
                            ON FCT_CONTRATO.NRSEQCONTRATO = GCHK.NRSEQCONTRATO
                        WHERE GCHK.CDOPERADOR IN ({$hierarchyParam})
                    ORDER BY DTHORAAGEND";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function getConformities($parametros)
    {

        $sql = "SELECT NRCRITERIOTARGERCHK, NMCRITERIOTARGERCHK, IDMARCADO
                  FROM FCM_CRITERIOTARGERCHK
                 WHERE NRTAREFAGERCHK = :NRTAREFAGERCHK";

        return $this->connection->executeQuery($sql, $parametros)->fetchAllAssociative();
    }

    // Busca critérios
    public function getCriteria($params)
    {

        $sql = "SELECT FCM_CRITERIOTARGERCHK.NRCRITERIOTARGERCHK, FCM_CRITERIOTARGERCHK.NMCRITERIOTARGERCHK, FCM_CRITERIOTARGERCHK.IDMARCADO,
                       FCM_CHECKLISTAGEND.IDSITUACAOAGEND
                FROM FCM_CHECKLISTAGEND
                JOIN FCM_GERENCIAMENTOCHK
                ON FCM_GERENCIAMENTOCHK.NRCHECKLISTAGEND = FCM_CHECKLISTAGEND.NRCHECKLISTAGEND
                JOIN FCM_TAREFAGERCHK
                ON FCM_TAREFAGERCHK.NRGERENCIAMENTOCHK = FCM_GERENCIAMENTOCHK.NRGERENCIAMENTOCHK
                JOIN FCM_CRITERIOTARGERCHK
                ON FCM_CRITERIOTARGERCHK.NRTAREFAGERCHK = FCM_TAREFAGERCHK.NRTAREFAGERCHK
                WHERE FCM_CRITERIOTARGERCHK.NRTAREFAGERCHK = {$params}";

        return $this->connection->executeQuery($sql, $params)->fetchAllAssociative();
    }

    // Busca Observações
    public function getInspectionToUpdate($params)
    {

        $sql = " SELECT FCM_TAREFAGERCHK.NRTAREFAGERCHK, FCM_TAREFAGERCHK.NRGERENCIAMENTOCHK, FCM_TAREFAGERCHK.DSOBSTAREFAGERCHK, FCM_GERENCIAMENTOCHK.IDSITUACAOGERCHK, FCM_CHECKLISTAGEND.IDSITUACAOAGEND
                      FROM FCM_TAREFAGERCHK
                      JOIN FCM_TAREFACHK
                      ON FCM_TAREFACHK.NRTAREFACHK = FCM_TAREFAGERCHK.NRTAREFACHK
                      JOIN FCM_GERENCIAMENTOCHK
                      ON FCM_GERENCIAMENTOCHK.NRGERENCIAMENTOCHK = FCM_TAREFAGERCHK.NRGERENCIAMENTOCHK
                      JOIN FCM_CHECKLISTAGEND
                      ON FCM_CHECKLISTAGEND.NRCHECKLISTAGEND = FCM_GERENCIAMENTOCHK.NRCHECKLISTAGEND
                      WHERE FCM_TAREFAGERCHK.NRGERENCIAMENTOCHK = :NRGERENCIAMENTOCHK
                      AND FCM_TAREFACHK.NRATIVIDADECHK = :NRATIVIDADECHK";

        return $this->connection->executeQuery($sql, $params)->fetchAllAssociative();
    }

    // Busca Atividades
    public function getActivities($id)
    {
        $sql = "SELECT DISTINCT A.NRATIVIDADECHK, B.NMATIVIDADE
                    FROM FCM_ATIVIDADECHK A
                    JOIN FCM_ATIVIDADE B
                        ON B.NRATIVIDADE = A.NRATIVIDADE
                    JOIN FCM_TAREFACHK C
                        ON C.NRATIVIDADECHK = A.NRATIVIDADECHK
                        AND NRTAREFACHK IN (SELECT NRTAREFACHK
                                                FROM FCM_TAREFAGERCHK
                                             WHERE NRGERENCIAMENTOCHK = {$id})";

        return $this->connection->executeQuery($sql)->fetchAllAssociative();
    }

    // Insere um novo critério
    public function insertCriterio($params)
    {
        $sql = "INSERT INTO FCM_CRITERIOTARGERCHK  (NRCRITERIOTARGERCHK, NRTAREFAGERCHK, IDATIVO,
                                                    DTINCLUSAO, NMCRITERIOTARGERCHK, IDMARCADO)
                    VALUES (:NRCRITERIOTARGERCHK, :NRTAREFAGERCHK, 'S', SYSDATE, :NMCRITERIOTARGERCHK, 'S')";

        $this->connection->executeStatement($sql, $params);
    }

    // Atualiza o campo IDMARCADO do critério para S ou N
    public function updateCriterio($params, $params2)
    {
        $sql = "UPDATE FCM_CRITERIOTARGERCHK
                  SET IDMARCADO = :IDMARCADO,
                    DTULTATU = SYSDATE
                  WHERE NRCRITERIOTARGERCHK IN (SELECT NRCRITERIOTARGERCHK
                                                    FROM FCM_CRITERIOTARGERCHK
                                                   WHERE NRTAREFAGERCHK IN (SELECT FCM_TAREFAGERCHK.NRTAREFAGERCHK
                                                    FROM FCM_TAREFAGERCHK
                                                    JOIN FCM_TAREFACHK
                                                      ON FCM_TAREFACHK.NRTAREFACHK = FCM_TAREFAGERCHK.NRTAREFACHK
                                                   WHERE FCM_TAREFAGERCHK.NRGERENCIAMENTOCHK = :NRGERENCIAMENTOCHK
                                                     AND FCM_TAREFACHK.NRATIVIDADECHK = :NRATIVIDADECHK))" . $params2;

        $this->connection->executeStatement($sql, $params);
    }

    // Atualiza o campo de observação da vistoria
    public function updateNote($params)
    {
        $sql = "UPDATE FCM_TAREFAGERCHK
                SET DSOBSTAREFAGERCHK = :DSOBSTAREFAGERCHK
                    WHERE NRTAREFAGERCHK IN (SELECT FCM_TAREFAGERCHK.NRTAREFAGERCHK
                                  FROM FCM_TAREFAGERCHK
                                  JOIN FCM_TAREFACHK
                                    ON FCM_TAREFACHK.NRTAREFACHK = FCM_TAREFAGERCHK.NRTAREFACHK
                                 WHERE FCM_TAREFAGERCHK.NRGERENCIAMENTOCHK = :NRGERENCIAMENTOCHK
                                   AND FCM_TAREFACHK.NRATIVIDADECHK = :NRATIVIDADECHK)";

        $this->connection->executeStatement($sql, $params);
    }

    public function upStatusInspection($parametros)
    {
        $sql = "UPDATE FCM_GERENCIAMENTOCHK
                SET IDSITUACAOGERCHK = :IDSITUACAOGERCHK,
                DTULTATU = SYSDATE
                  WHERE NRGERENCIAMENTOCHK = :NRGERENCIAMENTOCHK";


        $this->connection->executeStatement($sql, $parametros);
    }

    // Busca as informações de um determinado documento
    public function getDocumento(array $params)
    {
        $sql = "SELECT NMARQUIVO,
                  VRTAMANHO,
                  DSCAMINHOARQ,
                  DSTIPO
                FROM FCT_MODELODOC
                WHERE NRSEQMODDOCFCT = :NRSEQMODDOCFCT
                  AND NRORG          = :NRORG";
        return $this->connection->executeQuery($sql, $params)->fetchAllAssociative();
    }
}
