<?php

namespace FM\Service;

use FM\Factory\Fm00010000ParametrizacaoGeralFactory;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\NewCode;
use Doctrine\ORM\EntityManager;

class Fm00010000ParametrizacaoGeralService
{

    protected $ParametrizacaoGeralFactory;
    protected $Environment;
    protected $newCode;
    protected $connection;

    public function __construct(Fm00010000ParametrizacaoGeralFactory $ParametrizacaoGeralFactory, Environment $Environment, NewCode $newCode, EntityManager $entityManager)
    {
        $this->ParametrizacaoGeralFactory   = $ParametrizacaoGeralFactory;
        $this->Environment                  = $Environment;
        $this->newCode                      = $newCode;
        $this->connection                   = $entityManager->getConnection();
    }

    public function getParametrizacoes()
    {   
        return $this->ParametrizacaoGeralFactory->getParametrizacoes();
    }

    public function salvarParametrizacoes($params)
    {
        if ($params['PERGEOLOCALIZACAO'] == "N") {
            $params['INICIARTARGEOLOCALIZACAO'] = "N";
        }

        $this->connection->beginTransaction();
        try {
            $CDOPERADOR = $this->Environment->getCdOperador();
            $NRORG = $this->Environment->getNrOrg();
            
            $retorno = $this->ParametrizacaoGeralFactory->getParametrizacoes();
            $valida = isset($retorno[0]['NRPARAMGERAL']) ? $retorno[0]['NRPARAMGERAL'] : 'N';
             
            if($valida != 'N'){
                $param  = array(
                    "CDOPERULTATU"            => $CDOPERADOR,
                    "IDVALIDACRIATAREFA"      => $params['CONFLITO'],
                    "IDUTILIZAGEOLOCAL"       => $params['PERGEOLOCALIZACAO'],
                    "IDUTILIZAGEOLOCALINITAR" => $params['INICIARTARGEOLOCALIZACAO'],
                    "IDATIVO"                 => 'S',
                    'NRORGULTATU'             => $NRORG,
                    'CDOPERULTATU'            => $CDOPERADOR,
                );

                 $this->ParametrizacaoGeralFactory->updateParametrizacao($param);
            } else {
                $param  = array(
                    "NRPARAMGERAL"            => $this->newCode->getCodeByTableName("FCM_PARAMGERAL", $NRORG, 1, 10, true),
                    "IDVALIDACRIATAREFA"      => $params['CONFLITO'],
                    "IDUTILIZAGEOLOCAL"       => $params['PERGEOLOCALIZACAO'],
                    "IDUTILIZAGEOLOCALINITAR" => $params['INICIARTARGEOLOCALIZACAO'],
                    "IDATIVO"                 => 'S',
                    "NRORG"                   => $NRORG,
                    "CDOPERINCLUSAO"          => $CDOPERADOR,
                    "NRORGINCLUSAO"           => $NRORG
                );

                $this->ParametrizacaoGeralFactory->inserirParametrizacao($param);
            }

            $this->connection->commit();
            //$response = ["OK"];
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function getMascara()
    {
        return $this->ParametrizacaoGeralFactory->getMascara();
    }

    // Integração com SM
    public function salvarRelacionamento($parametros)
    {
        $this->connection->beginTransaction();

        try {
            $NRORG = $this->Environment->getNrOrg();
            $cdOperador = $this->Environment->getCdOperador();
            $contador = array();

            foreach ($parametros['ATIVIDADE'] as $item) {
                $retorno = $this->ParametrizacaoGeralFactory->validarItem(array(
                    "NRATIVIDADE"       => $item['NRATIVIDADE'],
                    "CDSERVORCA"        => $parametros['CDSERVORCA'],
                    "CDTPMEDIDASERV"    => $parametros['CDTPMEDIDASERV']
                ));

                if ($retorno == 0) {
                    $params = array(
                        "NRRELSERVITEMTAR"  => $this->newCode->getCodeByTableName("FCM_PARAMGERAL", $NRORG, 1, 10, true),
                        "NRATIVIDADE"       => $item['NRATIVIDADE'],
                        "CDSERVORCA"        => $parametros['CDSERVORCA'],
                        "CDTPMEDIDASERV"    => $parametros['CDTPMEDIDASERV'],
                        "NRORG"             => $NRORG,
                        "NRORGINCLUSAO"     => $NRORG,
                        "CDOPERINCLUSAO"    => $cdOperador
                    );

                    $this->ParametrizacaoGeralFactory->inserirRelacionamento($params);
                } else {
                    array_push($contador, $item['NMATIVIDADE']);
                }
            }
            $this->connection->commit();
            return $contador;
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function getRelacionamentoServicoTipoMedida()
    {
        return $this->ParametrizacaoGeralFactory->getRelacionamentoServicoTipoMedida();
    }

    public function getItensRelacionamentoServicoTipoMedida($parametros)
    {
        return $this->ParametrizacaoGeralFactory->getItensRelacionamentoServicoTipoMedida(array(
            "CDSERVORCA"        => $parametros['CDSERVORCA'],
            "CDTPMEDIDASERV"    => $parametros['CDTPMEDIDASERV'],
        ));
    }

    public function inativarServicos($parametros)
    {
        $this->connection->beginTransaction();
        try {
            $NRORG = $this->Environment->getNrOrg();
            $cdOperador = $this->Environment->getCdOperador();

            $this->ParametrizacaoGeralFactory->inativarServicos(array(
                "CDOPERULTATU"      => $cdOperador,
                "NRORGULTATU"       => $NRORG,
                "CDSERVORCA"        => $parametros['NRS'][0]['CDSERVORCA'],
                "CDTPMEDIDASERV"    => $parametros['NRS'][0]['CDTPMEDIDASERV']
            ));

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }
}
