/* eslint-disable */
import {
    Http, IDictionary, IEventParam, Metadata,
} from '@zeedhi/core';
import {
    DialogService, Text, LoadingService, Modal, ModalService, Select, TextInput,
} from '@zeedhi/common';
import { TekGrid, TekTreeGrid } from '@zeedhi/teknisa-components-common';
import { Utilities } from '@/util/Utilities';

export class Fm0000000HierarquiaUsuariosController {
    private basicModal!: Modal;

    public visibilidade: boolean = true;

    private nrhierarquia = 0;

    public tituloModal: String = 'Inserir Hierarquia';

    private clicks = 0;

    public disabled: boolean = false;

    private currentRow: any = '';

    public addNewRow({ component }: IEventParam<TekGrid>) {
        const newId = 0;
        component.addNewRow({ NRHIERARQUIAFUNC: newId, IDATIVO: 'N' }, 'start');
        setTimeout(() => {
            const nameInput = Metadata.getInstance<TextInput>(`NMHIERARQUIAFUNC_editable_${newId}`);
            nameInput.setFocus();
        }, 100);
    }

    get visibilidadeFrame() {
        return !this.visibilidade;
    }

    // Dar visibilidade  e exibir as informações no grid com a hierarquia escolhida
    public visualizarHierarquia({ row }: any) {

        this.visibilidade = false;
        this.nrhierarquia = row.NRHIERARQUIAFUNC;
        Metadata.getInstance<Text>('nomeHierarquia').text = row.NMHIERARQUIAFUNC;
        setTimeout(() => {
            Metadata.getInstance<TekTreeGrid>('tree-grid-hierarquia').datasource.setFilter({
                NRHIERARQUIAFUNC: this.nrhierarquia,
            });
        }, 1000);
    }

    //Setar o NR da Hierarquia para enviar pelo filtro do datasource para requisições get
    get getFindNr() {
        return { NRHIERARQUIAFUNC: this.nrhierarquia };
    }

    public voltarPaginaInicial() {
        this.visibilidade = true;
        setTimeout(() => {
            Metadata.getInstance<TekTreeGrid>('tree-grid-hierarquia').datasource.setFilter({
                NRHIERARQUIAFUNC: 0,
            });
        }, 100);
    }


    public setFiltroHierarquia({ component }: any) {
        component.datasource.setFilter({
            NRHIERARQUIAFUNC: this.nrhierarquia,
        });
    }

    // Função para inserir e editar hierarquias
    public editarHierarquia({ component, event }: any) {
        event.preventDefault();
        const valida = component.getEditedRows();

        if (valida.length > 0) {
            if (valida[0]["NMHIERARQUIAFUNC"] != null) {
                LoadingService.show();
                Http.post('/hierarquiaUsuarios', {
                    DRG: component.getEditedRows(),
                }).then(() => {
                    component.datasource.get();
                    Utilities.showSuccessMessage();
                }).catch((error: any) => {
                    Utilities.showErrorMessage(error);
                }).finally(() => {
                    component.cancelEditedRows();
                    LoadingService.hide();
                });
            } else {
                Utilities.alertMessage('Preencha o campo de "Hierarquia"!', 'error');
            }
        } else {
            Utilities.alertMessage('Preencha o campo de "Hierarquia"!', 'error');
        }
    }

    //Função para inserir e editar os usuários nas hierarquias
    public inserirUsuarios() {
        LoadingService.show();
        Http.post('/inserirUsuarios', {
            CDOPERADOR: Metadata.getInstance<Select>('CDOPERADOR').value,
            CDOPERADORPAI: Metadata.getInstance<Select>('CDOPERADORPAI').value,
            NRHIERARQUIAFUNC: this.nrhierarquia,
        }).then(() => {
            this.fecharModal();
            Metadata.getInstance<TekTreeGrid>('tree-grid-hierarquia').datasource.get();
            Utilities.showSuccessMessage();
        }).catch((error: any) => {
            if (error.response.data.error === 'Exception: IDATIVO != S') {
                Utilities.alertMessage('Não foi possível inserir um novo usuário', 'error');
            } else {
                Utilities.showErrorMessage(error);
            }
        }).finally(() => {
            LoadingService.hide();
        });

    }

    public validaIdAtivo({ component }: any) {
        // Entra no IF apena se a mudança no select for para IDATIVO "Sim" 
        if (component.value == 'S') {
            const DADOS_GRID = Metadata.getInstance<TekGrid>("basic-tek-grid").datasource.data; // Pega todos os dados do grid
            const NRS_DADOS = DADOS_GRID.map((item: any) => item.NRHIERARQUIAFUNC); // Retira dos dados do grid apenas o NR dos elementos
            const NRS_DADOS_SEM_ATUAL = NRS_DADOS.filter((nr: any) => nr != component.name.split("_")[2]); // Dos NRS retirados do grid, exclui apenas o que acabou de ser modificado
            const TODOS_SELECTS = NRS_DADOS_SEM_ATUAL.map((nr) => Metadata.getInstance<Select>(`IDATIVO_editable_${nr}`)); // Salva em um array todos os selects de IDATIVO que não seja o recem modificado
            TODOS_SELECTS.forEach((element: any) => {
                element.setValue('N');
            }); // Loop para percorrer todos os arrays para setar o valor "Não", não permitindo que haja mais de um ativo.
        }

    }

    public excluirHierarquia({ component, event }: any) {
        Utilities.showConfirmDeleteMessage(event, 'Fm0000000HierarquiaUsuariosController.aposConfirmarDelete', 'Excluir Registro');
    }

    public aposConfirmarDelete() {
        const component = Metadata.getInstance<TekGrid>("basic-tek-grid");
        const NRS = (component.selectedRows.map((element: any) => element.NRHIERARQUIAFUNC));

        LoadingService.show();
        Http.post('/excluirHierarquia', {
            NRS: NRS,
        }).then(() => {
            component.datasource.get();
            Utilities.showSuccessMessage();
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            component.selectedRows = [];
            LoadingService.hide();
            DialogService.hide()
        });
    }

    public excluirUsuarios({ component, event }: any) {
        Utilities.showConfirmDeleteMessage(event, 'Fm0000000HierarquiaUsuariosController.deleteUsuarios', 'Excluir Registros');
    }

    public deleteUsuarios({ component }: any) {
        const GRID = Metadata.getInstance<TekTreeGrid>("tree-grid-hierarquia");
        const NRS = (GRID.selectedRows.map((element: any) => element.CDOPERADOR));

        LoadingService.show();
        Http.post('/excluirUsuarios', {
            NRS: NRS,

        }).then(() => {
            GRID.datasource.get();
            Utilities.showSuccessMessage();
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            GRID.selectedRows = [];
            LoadingService.hide();
            DialogService.hide()
        });
    }

    public async exibirModal() {
        this.basicModal = await ModalService.createFromJson('fm_00008_000_hierarquia_usuarios/modal/fm_00008_001_modal_inserir_usuario', true);
        this.basicModal.show();
    }

    public async abrirModalInserir() {
        await this.exibirModal();
        this.tituloModal = 'Inserir Hierarquia';

        if (this.currentRow !== '') {
            Metadata.getInstance<Select>('CDOPERADORPAI').value = this.currentRow;
        }
    }

    public rowClick({ row }: any) {
        this.currentRow = row.CDOPERADOR;
    }

    public fecharModal() {
        this.basicModal.hide();
        this.basicModal?.destroy();
        this.disabled = false;
    }

    // função que simula um double click
    public rowClickFunction({ row }: any) {
        this.clicks += 1;
        setTimeout(() => {
            const grid = Metadata.getInstance<TekGrid>('basic-tek-grid');

            if (this.clicks == 1 && !grid.editing) {
                this.visualizarHierarquia({ row: row });
            }
            this.clicks = 0;
        },
            1000);
    }

}